@@ -3,7 +3,7 @@ package App::td;
 our $AUTHORITY = 'cpan:PERLANCAR'; # AUTHORITY
 our $DATE = '2020-05-29'; # DATE
 our $DIST = 'App-td'; # DIST
-our $VERSION = '0.096'; # VERSION
+our $VERSION = '0.097'; # VERSION
 
 use 5.010001;
 #IFUNBUILT
@@ -37,11 +37,15 @@ our %actions = (
     'sum-row' => {summary=>'Append a row containing sums'},
     'sum' => {summary=>'Return a row containing sum of all numeric columns'},
     'tail' => {summary=>'Only return the last N rows'},
-    'wc-row' => {summary=>'Alias for rowcount-row'},
-    'wc' => {summary=>'Alias for rowcount'},
+    'transpose' => {summary=>'Transpose table'},
+    'wc-row' => {summary=>'Alias for rowcount-row', tags=>['alias']},
+    'wc' => {summary=>'Alias for rowcount', tags=>['alias']},
 
-    'grep' => {summary=>'Use Perl code to filter rows', tags=>['perl']},
-    'map' => {summary=>'Use Perl code to transform row', tags=>['perl']},
+    'grep-row' => {summary=>'Use Perl code to filter rows', tags=>['perl']},
+    'grep' => {summary=>'Alias for grep-row', tags=>['perl', 'alias']},
+    'grep-col' => {summary=>'Use Perl code to filter columns', tags=>['perl']},
+    'map-row' => {summary=>'Use Perl code to transform rows', tags=>['perl']},
+    'map' => {summary=>'Alias for map-row', tags=>['perl', 'alias']},
     'psort' => {summary=>'Use Perl code to transform row', tags=>['perl']},
 );
 
@@ -212,23 +216,45 @@ Next, you can use these actions:
  # Show rows from the row 5 onwards
  % osnames -l --json | td tail -n +5
 
- # Use Perl code to filter rows. Perl code gets row in $ROW or $_
- # (scalar/aos/hos) or $ROW_HOS (always a hos) or $ROW_AOS (always aos). There
- # is also $ROW_NUM (integer, starts at 0). Perl code is eval'ed in the 'main'
- # package with strict/warnings turned off. The example below selects videos
- # that are larger than 480p.
+ # Transpose table (make first column of rows as column names in the transposed
+ # table)
+
+ % osnames -l --json | td transpose
+
+ # Transpose table (make columns named 'row1', 'row2', 'row3', ... in the
+ # transposed table)
+
+ % osnames -l --json | td transpose --no-header-column
+
+ # Use Perl code to filter rows. Perl code gets row in $row or $_
+ # (scalar/aos/hos) or $rowhash (always a hos) or $rowarray (always aos). There
+ # are also $rownum (integer, starts at 0) and $td (table data object). Perl
+ # code is eval'ed in the 'main' package with strict/warnings turned off. The
+ # example below selects videos that are larger than 480p.
+
  % media-info *.mp4 | td grep 'use List::Util qw(min); min($_->{video_height}, $_->{video_width}) > 480'
 
- # Use Perl code to transform row. Perl code gets row in $ROW or $_
+ # Use Perl code to filter columns. Perl code gets column name in $colname or
+ # $_. There's also $colidx (column index, from 1) and $td (table data object).
+ # If table data form is 'hash' or 'aos', it will be transformed into 'aoaos'.
+ # The example below only select even columns that match /col/i. Note that most
+ # of the time, 'td select' is better. But when you have a lot of columns and
+ # want to select them programmatically, you have grep-col.
+
+ % somecd --json | td grep-col '$colidx % 2 == 0 && /col/i'
+
+ # Use Perl code to transform row. Perl code gets row in $row or $_
  # (scalar/hash/array) and is supposed to return the new row. As in 'grep',
- # $ROW_HOS, $ROW_AOS, $ROW_NUM are also available as helper. The example below
- # adds a field called 'is_landscape'.
+ # $rowhash, $rowarray, $rownum, $td are also available as helper. The example
+ # below adds a field called 'is_landscape'.
+
  % media-info *.jpg | td map '$_->{is_landscape} = $_->{video_height} < $_->{video_width} ? 1:0; $_'
 
  # Use perl code to sort rows. Perl sorter code gets row in $a & $b or $_[0] &
  # $_[1] (hash/array). Sorter code, like in Perl's standard sort(), is expected
  # to return -1/0/1. The example belows sort videos by height, descendingly then
  # by width, descendingly.
+
  % media-info *.mp4 | td psort '$b->{video_height} <=> $a->{video_height} || $b->{video_width} <=> $b->{video_width}'
 
 _
@@ -276,6 +302,13 @@ _
             cmdline_aliases => {e=>{}},
             tags => ['category:select-action'],
         },
+
+        no_header_column => {
+            summary => "Don't make the first column as column names of the transposed table; ".
+                "instead create column named 'row1', 'row2', ...",
+            schema => 'true*',
+            tags => ['category:transpose-action'],
+        },
     },
 };
 sub td {
@@ -554,7 +587,7 @@ sub td {
             last;
         }
 
-        if ($action =~ /\A(grep|map|psort)\z/) {
+        if ($action =~ /\A(grep-row|grep|map-row|map|psort)\z/) {
             return [400, "Usage: td $action <perl-code>"] unless @$argv == 1;
             my $code_str;
             if ($action eq 'psort') {
@@ -574,21 +607,24 @@ sub td {
             my $input_rows_hos = $input_obj->rows_as_aohos;
 
             my $output_rows;
-            if ($action eq 'grep' || $action eq 'map') {
+            if ($action eq 'grep-row' || $action eq 'grep' ||
+                    $action eq 'map-row' || $action eq 'map'
+                ) {
                 for my $row_num (0 .. $#{ $input_rows }) {
                     my $code_res;
                     {
                         no warnings 'once';
-                        local $main::ROW_NUM = $row_num;
-                        local $_             = $input_rows->[$row_num];
-                        local $main::ROW     = $input_rows->[$row_num];
-                        local $main::ROW_AOS = $input_rows_aos->[$row_num];
-                        local $main::ROW_HOS = $input_rows_hos->[$row_num];
+                        local $_              = $input_rows->[$row_num];
+                        local $main::row      = $input_rows->[$row_num];
+                        local $main::rowarray = $input_rows_aos->[$row_num];
+                        local $main::rowhash  = $input_rows_hos->[$row_num];
+                        local $main::rownum   = $row_num;
+                        local $main::td       = $input_obj;
                         $code_res = $code->($_);
                     }
-                    if ($action eq 'grep') {
+                    if ($action eq 'grep-row' || $action eq 'grep') {
                         push @$output_rows, $input_rows->[$row_num] if $code_res;
-                    } else { # map
+                    } else { # map-row / map
                         push @$output_rows, $code_res;
                     }
                 }
@@ -600,6 +636,69 @@ sub td {
             last;
         }
 
+        if ($action =~ /\A(grep-col)\z/) {
+            return [400, "Usage: td $action <perl-code>"] unless @$argv == 1;
+            my $code_str = "package main; no strict; no warnings; sub { $argv->[0] }";
+            my $code = eval $code_str; die if $@;
+
+            my $input_cols = $input_obj->cols_by_idx;
+            my $output_cols = [];
+            for my $col_idx (0 .. $#{ $input_cols }) {
+                my $code_res;
+                {
+                    no warnings 'once';
+                    local $_              = $input_cols->[$col_idx];
+                    local $main::colname  = $input_cols->[$col_idx];
+                    local $main::colidx   = $col_idx;
+                    local $main::td       = $input_obj;
+                    $code_res = $code->($_);
+                }
+                push @$output_cols, $code_res ? $input_cols->[$col_idx] : undef;
+            }
+
+            if ($input_form eq 'hash' || $input_form eq 'aos') {
+                $input_obj = _get_td_obj($input_form->rows_as_aoaos);
+                $input_form = 'aoaos';
+            }
+            for my $col_idx (reverse 0..$#{ $output_cols }) {
+                unless (defined $output_cols->[$col_idx]) {
+                    $input_obj->del_col($col_idx);
+                }
+            }
+            $output = [
+                200,
+                "OK",
+                $input_form eq 'aohos' ? $input_obj->rows_as_aohos : $input_obj->rows_as_aoaos,
+                $input_form eq 'aohos' ? $input->[3] : undef
+            ];
+            last;
+        }
+
+        if ($action eq 'transpose') {
+            my $input_rows = $input_obj->rows_as_aoaos;
+            my $input_cols = $input_obj->cols_by_idx;
+
+            my @output_cols;
+            if ($args{no_header_column} || !@$input_rows) {
+                @output_cols = map {"row$_"} 1 .. @$input_rows;
+            } else {
+                @output_cols = map { $input_rows->[$_-1][0] } 1 .. @$input_rows;
+            }
+
+            my @output_rows;
+
+            for my $inputrowidx (0..$#{ $input_rows }) {
+                my $inputrow = $input_rows->[$inputrowidx];
+                for my $inputcolidx (0..$#{ $input_cols }) {
+                    $output_rows[$inputcolidx] //= [];
+                    $output_rows[$inputcolidx][$inputrowidx] = $inputrow->[$inputcolidx];
+                }
+            }
+
+            $output = [200, "OK", \@output_rows, {'table.fields'=>\@output_cols}];
+            last;
+        }
+
         return [400, "Unknown action '$action'"];
     } # PROCESS
 
@@ -631,7 +730,7 @@ App::td - Manipulate table data
 
 =head1 VERSION
 
-This document describes version 0.096 of App::td (from Perl distribution App-td), released on 2020-05-29.
+This document describes version 0.097 of App::td (from Perl distribution App-td), released on 2020-05-29.
 
 =head1 FUNCTIONS
 
@@ -751,23 +850,45 @@ Next, you can use these actions:
  # Show rows from the row 5 onwards
  % osnames -l --json | td tail -n +5
 
- # Use Perl code to filter rows. Perl code gets row in $ROW or $_
- # (scalar/aos/hos) or $ROW_HOS (always a hos) or $ROW_AOS (always aos). There
- # is also $ROW_NUM (integer, starts at 0). Perl code is eval'ed in the 'main'
- # package with strict/warnings turned off. The example below selects videos
- # that are larger than 480p.
+ # Transpose table (make first column of rows as column names in the transposed
+ # table)
+
+ % osnames -l --json | td transpose
+
+ # Transpose table (make columns named 'row1', 'row2', 'row3', ... in the
+ # transposed table)
+
+ % osnames -l --json | td transpose --no-header-column
+
+ # Use Perl code to filter rows. Perl code gets row in $row or $_
+ # (scalar/aos/hos) or $rowhash (always a hos) or $rowarray (always aos). There
+ # are also $rownum (integer, starts at 0) and $td (table data object). Perl
+ # code is eval'ed in the 'main' package with strict/warnings turned off. The
+ # example below selects videos that are larger than 480p.
+
  % media-info *.mp4 | td grep 'use List::Util qw(min); min($I<< ->{video >>height}, $I<< ->{video >>width}) > 480'
 
- # Use Perl code to transform row. Perl code gets row in $ROW or $_
+ # Use Perl code to filter columns. Perl code gets column name in $colname or
+ # $_. There's also $colidx (column index, from 1) and $td (table data object).
+ # If table data form is 'hash' or 'aos', it will be transformed into 'aoaos'.
+ # The example below only select even columns that match /col/i. Note that most
+ # of the time, 'td select' is better. But when you have a lot of columns and
+ # want to select them programmatically, you have grep-col.
+
+ % somecd --json | td grep-col '$colidx % 2 == 0 && /col/i'
+
+ # Use Perl code to transform row. Perl code gets row in $row or $_
  # (scalar/hash/array) and is supposed to return the new row. As in 'grep',
- # $ROW_HOS, $ROW_AOS, $ROW_NUM are also available as helper. The example below
- # adds a field called 'is_landscape'.
+ # $rowhash, $rowarray, $rownum, $td are also available as helper. The example
+ # below adds a field called 'is_landscape'.
+
  % media-info *.jpg | td map '$I<< ->{is >>landscape} = $I<< ->{video >>height} < $I<< ->{video >>width} ? 1:0; $_'
 
  # Use perl code to sort rows. Perl sorter code gets row in $a & $b or $I<[0] &
  # $>[1] (hash/array). Sorter code, like in Perl's standard sort(), is expected
  # to return -1/0/1. The example belows sort videos by height, descendingly then
  # by width, descendingly.
+
  % media-info *.mp4 | td psort '$b->{video_height} <=> $a->{video_height} || $b->{video_width} <=> $b->{video_width}'
 
 This function is not exported.
@@ -790,6 +911,10 @@ Arguments.
 
 =item * B<lines> => I<str>
 
+=item * B<no_header_column> => I<true>
+
+Don't make the first column as column names of the transposed table; instead create column named 'row1', 'row2', ...
+
 =item * B<repeat> => I<bool>
 
 Allow duplicates.
